@@ -96,12 +96,15 @@ logic.sampling = function(fitted, event, evidence, n, batch, debug = FALSE) {
       cat("* generated", m, "samples from the bayesian network.\n")
 
     # evaluate the expression defining the evidence.
-    r = eval(evidence, generated.data, parent.frame())
+    if (identical(evidence, TRUE))
+      r = rep(TRUE, m)
+    else
+      r = eval(evidence, generated.data, parent.frame())
     # double check that this is a logical vector.
     if (!is.logical(r))
       stop("evidence must evaluate to a logical vector.")
     # double check that it is of the right length.
-    if ((length(r) != 1) && (length(r) != m))
+    if (length(r) != m)
       stop("logical vector for evidence is of length ", length(r),
         " instead of ", m, ".")
     # filter out the samples not matching the evidence we assume.
@@ -123,12 +126,15 @@ logic.sampling = function(fitted, event, evidence, n, batch, debug = FALSE) {
     }#THEN
 
     # evaluate the expression defining the event.
-    r = eval(event, generated.data, parent.frame())
+    if (identical(event, TRUE))
+      r = rep(TRUE, m)
+    else
+      r = eval(event, generated.data, parent.frame())
     # double check that this is a logical vector.
     if (!is.logical(r))
       stop("event must evaluate to a logical vector.")
     # double check that it is of the right length.
-    if ((length(r) != 1) && (length(r) != m))
+    if (length(r) != m)
       stop("logical vector for event is of length ", length(r),
         " instead of ", m, ".")
     # filter out the samples not matching the event we are looking for.
@@ -228,6 +234,9 @@ logic.distribution = function(fitted, nodes, evidence, n, batch, debug = FALSE)
 
   }#FOR
 
+  # reset the row names.
+  rownames(result) = NULL
+
   if (debug && (nbatches > 1)) 
     cat("* generated a grand total of", n, "samples.\n")
  
