@@ -96,13 +96,13 @@ fast.incremental.association = function(x, whitelist, blacklist, test,
 }#FAST.INCREMENTAL.ASSOCIATION
 
 fast.ia.markov.blanket = function(x, data, nodes, alpha, B, whitelist, blacklist,
-  backtracking = NULL, test, debug = FALSE) {
+  start = character(0), backtracking = NULL, test, debug = FALSE) {
 
   nodes = nodes[nodes != x]
   known.good = known.bad = c()
   whitelisted = nodes[sapply(nodes,
           function(y) { is.whitelisted(whitelist, c(x, y), either = TRUE) })]
-  mb = c()
+  mb = start
   insufficient.data = FALSE
 
   del.node = function(y, x, test) {
@@ -142,12 +142,15 @@ fast.ia.markov.blanket = function(x, data, nodes, alpha, B, whitelist, blacklist
     cat("----------------------------------------------------------------\n")
     cat("* learning the markov blanket of", x, ".\n")
 
+    if (length(start) > 0)
+      cat("* initial set includes '", mb, "'.\n")
+
   }#THEN
 
   # whitelisted nodes are included by default (if there's a direct arc
   # between them of course they are in each other's markov blanket).
   # arc direction is irrelevant here.
-  mb = whitelisted
+  mb = unique(c(mb, whitelisted))
   nodes = nodes[!(nodes %in% mb)]
   # blacklist is not checked, not all nodes in a markov blanket must be
   # neighbours.
@@ -220,7 +223,7 @@ fast.ia.markov.blanket = function(x, data, nodes, alpha, B, whitelist, blacklist
               association[node], ", obs/cell:", opc, ").\n")
 
           }#ELSE
-          cat("    > markov blanket now is '", c(mb, node), "'.\n")
+          cat("    > markov blanket (", length(mb) + 1, " nodes ) now is '", c(mb, node), "'.\n")
 
         }#THEN
 
