@@ -96,13 +96,13 @@ inter.incremental.association = function(x, whitelist, blacklist, test,
 }#INTER.INCREMENTAL.ASSOCIATION
 
 inter.ia.markov.blanket = function(x, data, nodes, alpha, B, whitelist, blacklist,
-  backtracking = NULL, test, debug = FALSE) {
+  start = character(0), backtracking = NULL, test, debug = FALSE) {
 
   nodes = nodes[nodes != x]
   culprit = known.good = known.bad = c()
   whitelisted = nodes[sapply(nodes,
           function(y) { is.whitelisted(whitelist, c(x, y), either = TRUE) })]
-  mb = c()
+  mb = start
   loop.counter = 1
   state = vector(5 * length(nodes), mode = "list")
 
@@ -143,12 +143,15 @@ inter.ia.markov.blanket = function(x, data, nodes, alpha, B, whitelist, blacklis
     cat("----------------------------------------------------------------\n")
     cat("* learning the markov blanket of", x, ".\n")
 
+    if (length(start) > 0)
+      cat("* initial set includes '", mb, "'.\n")
+
   }#THEN
 
   # whitelisted nodes are included by default (if there's a direct arc
   # between them of course they are in each other's markov blanket).
   # arc direction is irrelevant here.
-  mb = whitelisted
+  mb = unique(c(mb, whitelisted))
   nodes = nodes[!(nodes %in% mb)]
   # blacklist is not checked, not all nodes in a markov blanket must be
   # neighbours.
@@ -200,7 +203,7 @@ inter.ia.markov.blanket = function(x, data, nodes, alpha, B, whitelist, blacklis
         function(x) {  cat("    >", x, "has p-value", association[x], ".\n")})
       cat("    @", to.add, "included in the markov blanket ( p-value:",
         association[to.add], ").\n")
-      cat("    > markov blanket now is '", c(mb, to.add), "'.\n")
+      cat("    > markov blanket (", length(mb) + 1, " nodes ) now is '", c(mb, to.add), "'.\n")
 
     }#THEN
 
